
div.speaker {
  font-style:italic;
}

div.description {
  font-size:120%;
}

div.talktime {
  font-style:italic;
}

span.dates {
  font-size:80%;
  font-style:italic;
}

table.talk {
  width:100%;
}

.talktitle {
  float: left;
}

.attendbutton {
  margin-right:10px;
  color:#FFFFFF;
  font-size:10px;
  margin-top:-18px;
  text-align:center;
  cursor: pointer;
  width:170px;
}

/* #attend { */
/*   background-color:#2EB85C; */
/* } */

/* #attend:hover { */
/*   background-color:#2E965C; */
/* } */

/* #unattend { */
/*   background-color:#7D7D7D; */
/* } */

/* #unattend:hover { */
/*   background-color:#646464; */
/* } */

#attendanceComponent{
  float:left;
}

#attendinfo {
  font-size:11px;
  font-weight: bold;
}

#attendcount {
  margin-top:-4px;
  font-size:10px;
  font-weight: bold;
}

.toolbarButton{
  display:block;
  float:left;
}

#sponsorbox{
  margin-right:10px;
}

#sponsorbox .sideBoxBody{
  text-align:center;
  background-color:#FFF;
}

#sponsorbox p {
  padding:5px;
  text-align:left;
  font-size:1.1em;
  font-family:verdana;
}

#sponsorbox img {
  padding:10px;
}

.Gold img{
  max-width: 90%;
}

.Silver img{
  max-width: 80%;
}

.Bronze img{
  max-width: 70%;
}

.Media img{
  max-width: 70%;
}

.conferenceBox {
  margin-bottom:14px;
  padding:0px;
  background:white;
  border:1px solid #BBBBBB;
}

/* .conferenceBox a { */
/*   color:#001240; */
/* } */

/* .conferenceBox a:hover { */
/*   color:#D37E23; */
/* } */

.conferenceBox div {
  padding: 2px 20px 2px 60px;
  font-size:1.2em;
  font-family:verdana;
  font-weight: bold;
}

#confCallForPaper {
  background:white;
  background-image:url('icon_pen.gif');
  background-repeat:no-repeat;
  background-position:-20px 5px;
}
#confRegister {
  background:white;
  background-image:url('ticket-icon.png');
  background-repeat:no-repeat;
  background-position:-20px 5px;
}

#statreview {
  margin:10px;
  padding-left:10px;
  border-left-style:solid;
  border-left-width:5px;
  border-left-color:red;
}

#stat ul {
  list-style-type:circle;
  list-style-position:inside;
}

#stat ul li {
  background:none;
}

#statview ul li {
  list-style-type:square;
}

#conftitle h1{
  border-bottom: 0px;
}